package com.qmth.teachcloud.report.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.report.business.bean.result.*;
import com.qmth.teachcloud.report.business.entity.TAExamCourse;
import com.qmth.teachcloud.report.business.entity.TAExamCourseRecord;
import com.qmth.teachcloud.report.business.entity.TBExam;
import com.qmth.teachcloud.report.business.entity.TBPaper;
import com.qmth.teachcloud.report.business.enums.PublishStatusEnum;
import com.qmth.teachcloud.report.business.enums.SemesterEnum;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseMapper;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseRecordMapper;
import com.qmth.teachcloud.report.business.service.ReportCommonService;
import com.qmth.teachcloud.report.business.service.TAExamCourseService;
import com.qmth.teachcloud.report.business.service.TBExamService;
import com.qmth.teachcloud.report.business.service.TBPaperService;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考试课程维度分析表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-06-01
 */
@Service
public class TAExamCourseServiceImpl extends ServiceImpl<TAExamCourseMapper, TAExamCourse> implements TAExamCourseService {

    @Resource
    TAExamCourseMapper taExamCourseMapper;

    @Resource
    TAExamCourseRecordMapper taExamCourseRecordMapper;

    @Resource
    ReportCommonService reportCommonService;

    @Resource
    TBPaperService tbPaperService;

    @Resource
    SysOrgService sysOrgService;

    @Resource
    TBExamService tbExamService;

    /**
     * 开课课程考试总览列表接口
     *
     * @param iPage
     * @param examId
     * @param semester
     * @param schoolId
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyTeacherList(IPage<Map> iPage, Long examId, SemesterEnum semester, Long schoolId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long collegeId = sysUser.getOrgId();
        return taExamCourseMapper.surveyTeacherList(iPage, examId, Objects.nonNull(semester) ? semester.name() : null, schoolId, collegeId);
    }


    @Override
    public ExamCourseResult getOverview(SemesterEnum semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return taExamCourseMapper.getOverview(schoolId, Objects.nonNull(semester) ? semester.name() : null, examId, courseCode);
    }

    /**
     * 赋分管理列表接口
     *
     * @param iPage
     * @param schoolId
     * @param examId
     * @param courseCode
     * @param publishStatus
     * @param semester
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyAspointsList(IPage<Map> iPage, Long schoolId, Long examId, String courseCode, PublishStatusEnum publishStatus, SemesterEnum semester) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        List<Long> collegeIds = sysOrgService.findCollegeIds(sysUser.getOrgId());
        if (Objects.nonNull(collegeIds) && collegeIds.size() > 0) {
            return taExamCourseMapper.surveyAspointsList(iPage, schoolId, examId, courseCode, Objects.nonNull(publishStatus) ? publishStatus.name() : null, Objects.nonNull(semester) ? semester.name() : null, collegeIds);
        } else {
            return taExamCourseMapper.surveyAspointsList(iPage, schoolId, examId, courseCode, Objects.nonNull(publishStatus) ? publishStatus.name() : null, Objects.nonNull(semester) ? semester.name() : null, Arrays.asList(sysUser.getOrgId()));
        }
    }

    @Override
    public Map<String, Object> getGradeDistribute(SemesterEnum semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, Object> map = new HashMap<>();
        //取总体和应届平均分
        SurveyTeacherExamCourseResult surveyTeacherExamCourseResult = reportCommonService.findAvgScore(schoolId, courseCode, examId);
        map.put("scoreAndRate", surveyTeacherExamCourseResult);
        //学院学科报表科目描述
        List<Map<String, Object>> mapList = new ArrayList<>();
        List<SurveyTeacherGradeDistributionResult> surveyTeacherGradeDistributionResultList = reportCommonService.surveyTeacherDistribution(examId, courseCode, sysUser.getOrgId());
        for (SurveyTeacherGradeDistributionResult s : surveyTeacherGradeDistributionResultList) {
            Map<String, Object> objectMap = new HashMap<>();
            objectMap.put("scores", String.join(",", s.getDescribe()));
            objectMap.put("totalRate", s.getAllCountRateAfter());
            objectMap.put("totalRealityCount", s.getAllCountAfter());
            objectMap.put("currentRealityCount", s.getCurrentCountAfter());
            objectMap.put("currentRate", s.getCurrentCountRateAfter());
            mapList.add(objectMap);
        }
        map.put("grades", mapList);
        return map;
    }

    /**
     * 考查课程考试分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.COLLEGE_COURSE_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode")
    public CollegeAndCourseResult surveyInspectAnalyseView(Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        //学院学科报表查询科目信息
        CourseInfoResult courseInfoResult = reportCommonService.findCourseInfo(examId, courseCode, sysUser.getOrgId());
        //学院学科报表科目描述
        CollegeAndSchoolGradeDistributionResult collegeAndSchoolGradeDistributionResult = reportCommonService.findCourseDistribution(examId, courseCode, sysUser.getOrgId());
        //学院学科报表维度查询
        DimensionAnalyzeResult dimensionAnalyzeResult = reportCommonService.findDimensionInfo(examId, courseCode, sysUser.getOrgId(), null);
        //查找题目相关
        QuestionInfoResult questionInfoResult = reportCommonService.findSituationOfQuestions(examId, courseCode, sysUser.getOrgId(), null);
        //查找老师得分
        TeacherInfoListResult teacherInfoListResult = reportCommonService.findTeacherInfo(examId, courseCode, sysUser.getOrgId());
        //查找题目信息
        QuestionListResult questionListResult = reportCommonService.findQuestionInfo(examId, courseCode, sysUser.getOrgId());

        return new CollegeAndCourseResult(courseInfoResult, collegeAndSchoolGradeDistributionResult, dimensionAnalyzeResult,
                questionInfoResult, teacherInfoListResult, questionListResult);
    }

    /**
     * 开课课程考试总览图表接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SURVEY_TEACHER_VIEW_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode")
    public SurveyTeacherViewResult surveyTeacherView(Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        // 考试概况
        TeachCourseSurveyResult teachCourseSurveyResult = this.getTeachCourseSurveyInfo(semester, examId, courseCode);

        //取总体和应届平均分
        SurveyTeacherExamCourseResult surveyTeacherExamCourseResult = reportCommonService.findAvgScore(schoolId, courseCode, examId);

        //学院学科报表科目描述
        List<SurveyTeacherGradeDistributionResult> surveyTeacherGradeDistributionResultList = reportCommonService.surveyTeacherDistribution(examId, courseCode, sysUser.getOrgId());
        //查找题目信息
        QuestionListResult questionListResult = reportCommonService.findQuestionInfo(examId, courseCode, sysUser.getOrgId());
        return new SurveyTeacherViewResult(surveyTeacherGradeDistributionResultList, questionListResult, surveyTeacherExamCourseResult,teachCourseSurveyResult);
    }

    /**
     * 开课课程考试总览-教师各课堂成绩排名接口
     *
     * @param iPage
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    public IPage<TAExamCourseCollegeTeacherResult> surveyTeacherTeacherView(IPage<Map> iPage, Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        return taExamCourseMapper.surveyTeacherTeacherView(iPage, schoolId, Objects.nonNull(semester) ? semester.name() : null, examId, courseCode);
    }

    /**
     * 开课课程考试总览-试题难度得分情况分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @param collegeId
     * @param teacherId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.TEACHER_VIEW_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode + '-' + #collegeId + '-' + #teacherId")
    public DimensionAnalyzeResult surveyTeacherQuestionView(Long schoolId, SemesterEnum semester, Long examId, String courseCode, Long collegeId, Long teacherId) {
        return reportCommonService.findDimensionInfo(examId, courseCode, collegeId, teacherId);
    }

    /**
     * 开课课程考试总览-各难度水平题目上的作答分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @param collegeId
     * @param teacherId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.TEACHER_ANSWER_VIEW_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode + '-' + #collegeId + '-' + #teacherId")
    public QuestionInfoResult surveyTeacherQuestionAnswerView(Long schoolId, SemesterEnum semester, Long examId, String courseCode, Long collegeId, Long teacherId) {
        return reportCommonService.findSituationOfQuestions(examId, courseCode, collegeId, teacherId);
    }

    @Override
    public TrialCalculationResult trialCalculate(Long examId, String courseCode, BigDecimal coefficient) {
        // 试卷分数设置
        List<TBPaper> tbPaperList = tbPaperService.list(new QueryWrapper<TBPaper>().lambda().eq(TBPaper::getExamId, examId).eq(TBPaper::getCourseCode, courseCode));
        if (tbPaperList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("该课程不存在");
        }
        TBPaper tbPaper = tbPaperList.get(0);
        BigDecimal fullScore = tbPaper.getTotalScore();
        BigDecimal passScore = tbPaper.getPassScore();
        // 参考的考生数据
        List<TAExamCourseRecord> taExamCourseRecordList = taExamCourseRecordMapper.selectList(new QueryWrapper<TAExamCourseRecord>().lambda()
                .eq(TAExamCourseRecord::getExamId, examId)
                .eq(TAExamCourseRecord::getCourseCode, courseCode)
                .eq(TAExamCourseRecord::getAbsent, false));
        for (TAExamCourseRecord taExamCourseRecord : taExamCourseRecordList) {
            BigDecimal paperScore = taExamCourseRecord.getTotalScore();
            BigDecimal assignScore;
            if (coefficient.compareTo(BigDecimal.ZERO) != 0) {
                assignScore = paperScore.add((fullScore.subtract(paperScore)).divide(coefficient, 4, BigDecimal.ROUND_HALF_UP));
            } else {
                assignScore = paperScore;
            }

            assignScore = assignScore.setScale(0, BigDecimal.ROUND_HALF_UP);
            taExamCourseRecord.setAssignedScore(assignScore);
        }
        int totalCount = taExamCourseRecordList.size();
        int currentCount = (int) taExamCourseRecordList.stream().filter(TAExamCourseRecord::getStudentCurrent).count();

        // 查找卷面数据
        // 总体
        List<Double> paperTotalScoreList = taExamCourseRecordList.stream().map(e -> e.getTotalScore().doubleValue()).collect(Collectors.toList());
        DoubleSummaryStatistics paperTotalStatistics = paperTotalScoreList.stream().collect(Collectors.summarizingDouble(e -> e));
        double paperTotalAvgScore = paperTotalStatistics.getAverage();
        long paperTotalFailCount = paperTotalScoreList.stream().filter(e -> e < passScore.doubleValue()).count();
        BigDecimal paperTotalFailRate = BigDecimal.valueOf(paperTotalFailCount).divide(BigDecimal.valueOf(paperTotalScoreList.size()), 4, BigDecimal.ROUND_HALF_UP);

        // 应届
        List<Double> paperCurrentScoreList = taExamCourseRecordList.stream().filter(TAExamCourseRecord::getStudentCurrent).map(e -> e.getTotalScore().doubleValue()).collect(Collectors.toList());
        DoubleSummaryStatistics paperCurrentStatistics = paperCurrentScoreList.stream().collect(Collectors.summarizingDouble(e -> e));
        double paperCurrentAvgScore = paperCurrentStatistics.getAverage();
        long paperCurrentFailCount = paperCurrentScoreList.stream().filter(e -> e < passScore.doubleValue()).count();
        BigDecimal paperCurrentFailRate = BigDecimal.valueOf(paperCurrentFailCount).divide(BigDecimal.valueOf(paperCurrentScoreList.size()), 4, BigDecimal.ROUND_HALF_UP);

        // 查找赋分试算后数据
        // 总体
        List<Double> totalScoreList = taExamCourseRecordList.stream().map(e -> e.getAssignedScore().doubleValue()).collect(Collectors.toList());
        DoubleSummaryStatistics totalStatistics = totalScoreList.stream().collect(Collectors.summarizingDouble(e -> e));
        double totalAvgScore = totalStatistics.getAverage();
        long totalFailCount = totalScoreList.stream().filter(e -> e < passScore.doubleValue()).count();
        BigDecimal totalFailRate = BigDecimal.valueOf(totalFailCount).divide(BigDecimal.valueOf(totalScoreList.size()), 4, BigDecimal.ROUND_HALF_UP);

        // 应届
        List<Double> currentScoreList = taExamCourseRecordList.stream().filter(TAExamCourseRecord::getStudentCurrent).map(e -> e.getAssignedScore().doubleValue()).collect(Collectors.toList());
        DoubleSummaryStatistics currentStatistics = currentScoreList.stream().collect(Collectors.summarizingDouble(e -> e));
        double currentAvgScore = currentStatistics.getAverage();
        long currentFailCount = currentScoreList.stream().filter(e -> e < passScore.doubleValue()).count();
        BigDecimal currentFailRate = BigDecimal.valueOf(currentFailCount).divide(BigDecimal.valueOf(currentScoreList.size()), 4, BigDecimal.ROUND_HALF_UP);

        // 组装
        TrialCalculationResult trialCalculationResult = new TrialCalculationResult();
        trialCalculationResult.setTotalCount(totalCount);
        trialCalculationResult.setCurrentCount(currentCount);
        // - 卷面-总体
        trialCalculationResult.setAvgScore(BigDecimal.valueOf(paperTotalAvgScore).setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setFailRate(paperTotalFailRate.setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setFailCount((int) paperTotalFailCount);
        // - 卷面-应届
        trialCalculationResult.setCurrentAvgScore(BigDecimal.valueOf(paperCurrentAvgScore).setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setCurrentFailRate(paperCurrentFailRate.setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setCurrentFailCount((int) paperCurrentFailCount);
        // - 赋分-总体
        trialCalculationResult.setAvgScoreAssign(BigDecimal.valueOf(totalAvgScore).setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setFailRateAssign(totalFailRate.setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setFailCountAssign((int) totalFailCount);
        // - 赋分-应届
        trialCalculationResult.setCurrentAvgScoreAssign(BigDecimal.valueOf(currentAvgScore).setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setCurrentFailRateAssign(currentFailRate.setScale(1, BigDecimal.ROUND_HALF_UP));
        trialCalculationResult.setCurrentFailCountAssign((int) currentFailCount);

        return trialCalculationResult;
    }


    /**
     * 获取开课课程考试课程总览
     * @param semester 学期
     * @param examId 考试id
     * @param courseCode 课程编号
     * @return 结果
     */
    private TeachCourseSurveyResult getTeachCourseSurveyInfo(SemesterEnum semester, Long examId, String courseCode) {
        TBExam tbExam = tbExamService.getById(examId);

        TAExamCourse taExamCourse = this.getOne(new QueryWrapper<TAExamCourse>().lambda().eq(TAExamCourse::getExamId,examId).eq(TAExamCourse::getCourseCode,courseCode));
        TeachCourseSurveyResult teachCourseSurveyResult = new TeachCourseSurveyResult();
        teachCourseSurveyResult.setExamTime(tbExam.getExamTime());
        teachCourseSurveyResult.setCourseCode(taExamCourse.getCourseCode());
        teachCourseSurveyResult.setCourseName(taExamCourse.getCourseName());
        teachCourseSurveyResult.setFullScore(taExamCourse.getTotalScore());
        teachCourseSurveyResult.setRealityCount(taExamCourse.getRealityCount());
        teachCourseSurveyResult.setInspectCollegeCount(taExamCourse.getInspectCollegeCount());
        teachCourseSurveyResult.setAbsentCount(taExamCourse.getAbsentCount());
        teachCourseSurveyResult.setCoefficient(taExamCourse.getCoefficient());
        teachCourseSurveyResult.setTeachCollegeName(taExamCourse.getTeachCollegeName());
        teachCourseSurveyResult.setInspectCollegeNames(taExamCourse.getInspectCollegeNames());
        return teachCourseSurveyResult;
    }
}
