package com.qmth.teachcloud.report.business.templete.execute;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.service.TBTaskService;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.report.business.entity.TBExamCourse;
import com.qmth.teachcloud.report.business.enums.PublishStatusEnum;
import com.qmth.teachcloud.report.business.service.AnalyzeForReportService;
import com.qmth.teachcloud.report.business.service.TBExamCourseService;
import com.qmth.teachcloud.report.business.service.TBExamService;
import com.qmth.teachcloud.report.business.templete.calculate.AsyncCalculateTaskTemplete;
import com.qmth.teachcloud.report.business.templete.service.TaskCalculateService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;

/**
 * @Description:
 * @Author: CaoZixuan
 * @Date:
 */
@Service
public class AsyncDataCalculateTempleteService extends AsyncCalculateTaskTemplete {
    @Resource
    private TBExamService tbExamService;
    private final static Logger log = LoggerFactory.getLogger(AsyncDataCalculateTempleteService.class);
    @Override
    public Result calculateTask(Map<String, Object> map) {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        tbTask.setStatus(TaskStatusEnum.RUNNING);

        TBTaskService tbTaskService = SpringContextHolder.getBean(TBTaskService.class);
        tbTaskService.updateById(tbTask);

        AnalyzeForReportService analyzeForReportService = SpringContextHolder.getBean(AnalyzeForReportService.class);
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        String courseCode = String.valueOf(map.get("courseCode"));

        TBExamCourseService tbExamCourseService = SpringContextHolder.getBean(TBExamCourseService.class);
        TBExamCourse tbExamCourse = tbExamCourseService.getOne(new QueryWrapper<TBExamCourse>().lambda()
                .eq(TBExamCourse::getExamId,examId)
                .eq(TBExamCourse::getCourseCode,courseCode));
        PublishStatusEnum oldStatus =  tbExamCourse.getPublishStatus();


        try {
            analyzeForReportService.normalAssignScore(examId,tbExamService.getById(examId).getSchoolId(),courseCode);
            analyzeForReportService.realityForCalculate(examId,courseCode);
            TaskCalculateService taskCalculateService = SpringContextHolder.getBean(TaskCalculateService.class);
            taskCalculateService.dataCalculate(map);
            tbExamCourse.setPublishStatus(PublishStatusEnum.UN_PUBLISH);
        }catch (Exception e){
            log.error("请求出错", e);
            tbTask.setRemark(e.getMessage());
            tbExamCourse.setPublishStatus(oldStatus);
        }finally {
            tbExamCourseService.updateById(tbExamCourse);
        }
        return ResultUtil.ok(map);
    }
}
